// 共通処理
let tabId = null;
async function getTabId() {
    if (tabId !== null) {
        //console.log("return cached tabId:", tabId);
        return tabId;
    }

    // Chrome runtime APIのsendMessageもPromiseを返すので、awaitで結果を待つ
    const response = await new Promise(resolve => {
        chrome.runtime.sendMessage({ type: 'getTabId' }, function (response) {
            resolve(response);
        });
    });

    // レスポンスからtabIdを取得
    tabId = response.tabId;
    //console.log("return tabId:", tabId);
    return tabId;
}


async function extractRepeatCountAndInterval(url) {
    let useUpAbility = (await getUseUpAbility()) &&
        url.search(/aid%/) >= 0 && 
        url.search(/aid%3D[1-6]0%/) < 0 &&
        url.search(/aid%3D4/) < 0
    console.log("url:", url);
    console.log("useUpAbility:", useUpAbility);

    // 攻撃間隔(設定値)
    const settingId = await getSettingId(await getTabId());
    const excuteInterval = await getExcuteInterval(settingId);
    console.log("sid:", settingId);
    console.log("excuteInterval:", excuteInterval);

    if (url) {
        //console.log("url=", url);
        const parts = url.split("#");
        if (parts.length === 2) {
            const countsAndInterval = parts[0].split("-");
            //console.log("len=", countsAndInterval.length);
            if (countsAndInterval.length === 2) {
                const repeatCount = !isNaN(countsAndInterval[0]) ? parseInt(countsAndInterval[0], 10) : 1;
                const interval = !isNaN(countsAndInterval[1]) ? parseInt(countsAndInterval[1], 10) : excuteInterval;
                if (useUpAbility) {
                    return { repeatCount: 999, interval };
                }
                return { repeatCount, interval };
            } else if (countsAndInterval.length === 1) {
                const repeatCount = !isNaN(countsAndInterval[0]) ? parseInt(countsAndInterval[0], 10) : 1;
                console.log("url.search: 3D[1-6]0%", url.search(/3D[1-6]0%/));
                // intervalが指定されていない場合、設定画面での設定値を返す
                if (useUpAbility) {
                    return { repeatCount: 999, interval: excuteInterval };
                }
                return { repeatCount, interval: excuteInterval };
            }
        }
    }
    // デフォルト値を返す
    if (useUpAbility) {
        return { repeatCount: 999, interval: excuteInterval };
    }
    return { repeatCount: 1, interval: excuteInterval };
}


// 文字列から数字を抽出する関数
function extractNumber(str) {
    const matches = str.match(/\d+/);
    return matches ? parseInt(matches[0], 10) : null;
}


// URLからrepeatCountを抽出し、#以降の部分を削除する関数
function extractRepeatCount(url) {
    const parts = url.split("#");
    if (parts.length === 2 && !isNaN(parts[0])) {
        return parseInt(parts[0], 10);
    } else {
        return 1;
    }
}


async function saveSettingIdToStorage(tabId, settingId) {
    return new Promise((resolve) => {
        chrome.storage.local.set({ [`settingId_${tabId}`]: settingId }, () => {
            resolve();
        });
    });
}


// 変数を保持するためのオブジェクト
const cachedSettingIds = {};
async function getSettingId(tabId) {
    // キャッシュ内に値があるか確認
    if (cachedSettingIds[tabId] !== undefined) {
        // キャッシュ内にある場合はそれを返す
        return cachedSettingIds[tabId];
    }

    // キャッシュ内に値がない場合はストレージから取得
    return new Promise((resolve) => {
        chrome.storage.local.get([`settingId_${tabId}`], (data) => {
            const settingId = data[`settingId_${tabId}`] !== undefined ? data[`settingId_${tabId}`] : 1;
            // 取得した値を変数に保存
            cachedSettingIds[tabId] = settingId;

            // 該当のキーが存在しない場合、デフォルトの settingId を保存
            if (data[`settingId_${tabId}`] === undefined) {
                saveSettingIdToStorage(tabId, 1);
            }

            resolve(settingId);
        });
    });
}


// タブの状態を読み出す関数
function loadTabState(tabId) {
    return new Promise(resolve => {
        chrome.storage.local.get('tabStates', function (result) {
            const tabStates = result.tabStates || {};
            const tabState = tabStates[tabId] || null;
            // タブの状態が存在しない場合、デフォルトの状態を設定
            const defaultTabState = {
                status: 0,
                retryCount: 0,
                url: '', // 修正: URL のキーを正しく指定
                waitTime: 0
            };
            const loadedTabState = tabState || defaultTabState;
            //console.log(`Tab state loaded for tabId ${tabId}:`, loadedTabState);
            resolve(loadedTabState);
        });
    });
}


// タブの状態を保存する関数
async function saveTabState(tabId, state) {
    // 現在の時刻を取得
    const timestamp = new Date().getTime();
    return new Promise((resolve) => {
        chrome.storage.local.get('tabStates', function (result) {
            const tabStates = result.tabStates || {};
            // stateオブジェクトにtimestampプロパティを追加
            state.timestamp = timestamp;
            tabStates[tabId] = state;
            chrome.storage.local.set({ 'tabStates': tabStates }, function () {
                console.log(`Tab state saved for tabId ${tabId}:`, state);
                resolve();
            });
        });
    });
}


// タブの状態を初期化する関数
async function clearTabState() {
    const tabId = await getTabId();

    // デフォルトの状態を設定
    const defaultTabState = {
        status: 0,
        retryCount: 0,
        url: '',
        waitTime: 0
    };

    // 現在の時刻を取得
    const timestamp = new Date().getTime();
    // タブの状態を初期状態に設定
    const initialState = {
        ...defaultTabState,
        timestamp: timestamp
    };

    return new Promise((resolve) => {
        chrome.storage.local.get('tabStates', function (result) {
            const tabStates = result.tabStates || {};
            tabStates[tabId] = initialState;

            chrome.storage.local.set({ 'tabStates': tabStates }, function () {
                //console.log(`Tab state cleared for tabId ${tabId}`);
                resolve();
            });
        });
    });
}


function getsettingName(settingId) {
    const storageKey = `sid${String(settingId).padStart(2, '0')}`;
    return new Promise(resolve => {
        chrome.storage.local.get({ [storageKey]: { settingName: "設定1" } }, obj => {
            resolve(obj[storageKey].settingName);
        });
    });
}


// 経過時間を取得する関数
async function getElapsedTime(tabState) {
    const currentTimestamp = new Date().getTime();
    const elapsedTime = currentTimestamp - tabState.timestamp;
    return elapsedTime;
}


function getRecoveryUrl(settingId) {
    const storageKey = `sid${String(settingId).padStart(2, '0')}`;
    return new Promise(resolve => {
        chrome.storage.local.get({ [storageKey]: { recoveryUrl: "https://sp.pf.mbga.jp/12010355/?guid=ON&url=https%3A%2F%2Fmbga-sp.daisenran.jp%2Fbattle%2Fability_execute.ashx%3Faid%3D30&guid=ON" } }, obj => {
            resolve(obj[storageKey].recoveryUrl);
        });
    });
}


// 攻撃間隔
function getExcuteInterval(settingId) {
    const storageKey = `sid${String(settingId).padStart(2, '0')}`;
    //console.log("storageKey:",storageKey);
    return new Promise(resolve => {
        chrome.storage.local.get({ [storageKey]: { executeInterval: 1000 } }, obj => {
            resolve(obj[storageKey]?.executeInterval || 1000);
        });
    });
}


// 回復⇒攻撃wait
function getWaitTime(settingId) {
    const storageKey = `sid${String(settingId).padStart(2, '0')}`;
    return new Promise(resolve => {
        chrome.storage.local.get({ [storageKey]: { waitTime: 300 } }, obj => {
            resolve(obj[storageKey].waitTime);
        });
    });
}


// 回復⇒攻撃wait
function getButtonSize(settingId) {
    const storageKey = `sid${String(settingId).padStart(2, '0')}`;
    return new Promise(resolve => {
        chrome.storage.local.get({ [storageKey]: { buttonSize: "medium" } }, obj => {
            resolve(obj[storageKey].buttonSize);
        });
    });
}

function getRecoveryOption(settingId) {
    const storageKey = `sid${String(settingId).padStart(2, '0')}`;
    return new Promise(resolve => {
        chrome.storage.local.get({ [storageKey]: { recoveryOption: "on" } }, obj => {
            resolve(obj[storageKey].recoveryOption);
        });
    });
}


async function getButtonSettings(buttonType, settingId) {
    const storageKey = `sid${String(settingId).padStart(2, '0')}`;
    const data = await new Promise(resolve => {
        chrome.storage.local.get({ [storageKey]: {} }, data => {
            resolve(data);
        });
    });
    return data[storageKey][`buttonSettings${buttonType}`] || [];
}


// ボタンの数を取得する関数
async function getButtonCount(buttonType, settingId) {
    const storageKey = `sid${String(settingId).padStart(2, '0')}`;
    const data = await new Promise(resolve => {
        chrome.storage.local.get({ [storageKey]: {} }, data => {
            resolve(data);
        });
    });
    return data[storageKey][`buttonCount${buttonType}`] || 10;
}


function createRadioButton(container) {
    const radioButtonWrapper = document.createElement("div");
    radioButtonWrapper.classList.add("radioButton");

    const addRadioButton = (label, value, checked) => {
        const radio = document.createElement("input");
        radio.type = "radio";
        radio.name = "recoveryOption";
        radio.value = value;
        radio.checked = checked;
        //radio.addEventListener("click", handleRecoveryOptionChange);

        const labelElement = document.createElement("label");
        labelElement.textContent = label;
        radioButtonWrapper.appendChild(radio);
        radioButtonWrapper.appendChild(labelElement);
    };

    chrome.storage.local.get({ recoveryOption: "on" }, obj => {
        addRadioButton("回復ON", "on", obj.recoveryOption === "on");
        addRadioButton("回復OFF", "off", obj.recoveryOption === "off");
    });

    container.appendChild(radioButtonWrapper);

    return radioButtonWrapper;
}


// ストレージに設定を保存する関数
async function setStorage(settingId, data) {
    const storageKey = `sid${String(settingId).padStart(2, '0')}`;
    return new Promise((resolve) => {
        chrome.storage.local.set({ [storageKey]: data }, () => {
            console.log(`Settings saved for ID ${settingId}:`, data);
            resolve();
        });
    });
}


// ストレージから設定を取得する関数
async function getStorage(settingId, keysAndDefaults) {
    const storageKey = `sid${String(settingId).padStart(2, '0')}`;
    return new Promise((resolve) => {
        chrome.storage.local.get({ [storageKey]: keysAndDefaults }, (obj) => {
            resolve(obj[storageKey] || keysAndDefaults);
        });
    });
}


// アビ使い切り設定保存
async function setUseUpAbility(useUp) {
    await new Promise((resolve) => {
        chrome.storage.local.set({ 'useUpAbility': useUp }, function () {
            //console.log('setUseUpAbility', useUp);
            resolve();
        });
    });
}


// アビ使い切り設定読み出し
async function getUseUpAbility() {
    return new Promise((resolve) => {
        chrome.storage.local.get(['useUpAbility'], function (result) {
            const useUpAbility = result.useUpAbility;
            //console.log('getUseUpAbility:', useUpAbility);
            resolve(useUpAbility);
        });
    });
}


async function bsleep(waitMsec) {
    var startMsec = new Date();
    //console.log("bsleep:", waitMsec);
    while (new Date() - startMsec < waitMsec) {
        await delay(10);
    }
}


async function delay(ms) {
    return new Promise(resolve => setTimeout(resolve, ms));
}


function displayMessage(message, success) {
    const containerWrapper = createBButtonsContainer(); // ボタンコンテナとメッセージコンテナを包む新しい要素を取得
    const statusMessageElement = document.createElement("div");
    statusMessageElement.id = "statusMessage";
    statusMessageElement.textContent = message;
    statusMessageElement.style.color = success ? "green" : "red";

    const bottomContainer = containerWrapper.querySelector("#bottomLinkButtons"); // bottomContainer を取得
    containerWrapper.insertBefore(statusMessageElement, bottomContainer); // メッセージコンテナをボタンコンテナの前に追加
    setTimeout(() => {
        statusMessageElement.textContent = "";
    }, 3000);
}


function displayStatus(message) {
    // ステータスメッセージ用の要素を取得または作成
    const statusElement = document.getElementById("statusTop") || document.createElement("div");
    statusElement.id = "statusTop";
    statusElement.textContent = message;

    // 要素が存在しない場合はbodyに追加
    if (!document.getElementById("statusTop")) {
        const body = document.body || document.documentElement;
        body.appendChild(statusElement);
    }
}


// 要素を削除する関数
function clearStatus() {
    const statusElement = document.getElementById("statusTop");
    if (statusElement) {
        statusElement.parentNode.removeChild(statusElement);
    }
}

