document.addEventListener("DOMContentLoaded", function () {
    function createElement(tag, attributes = {}, text) {
        const element = document.createElement(tag);
        Object.entries(attributes).forEach(([name, value]) => {
            // ブール値の属性は単に存在するだけで true になるので、値の有無だけをチェック
            if (value === true) {
                element.setAttribute(name, ''); // 値が true の場合は属性名だけ追加
            } else if (value !== false && value !== null && value !== undefined) {
                element.setAttribute(name, value);
            }
        });
        if (text) element.textContent = text;
        return element;
    }

    function createElementold(tag, attributes = {}, text) {
        const element = document.createElement(tag);
        Object.entries(attributes).forEach(([name, value]) => {
            element.setAttribute(name, value);
        });
        if (text) element.textContent = text;
        return element;
    }

    const recoveryOptionRadioGroup = document.getElementById("recoveryOptionRadioGroup");

    function addRecoveryOptionRadio(label, value, checked) {
        const radio = createElement("input", {
            type: "radio",
            name: "recoveryOption",
            value,
            checked
        });
        const labelElement = createElement("label", {}, label);

        const row = createElement("tr");
        const radioCell = createElement("td");
        const labelCell = createElement("td");
        radioCell.appendChild(radio);
        labelCell.appendChild(labelElement);
        row.appendChild(radioCell);
        row.appendChild(labelCell);
        recoveryOptionRadioGroup.appendChild(row);
    }

    function displayStatusMessage(message, success) {
        const statusMessageElement = document.getElementById("statusMessage");
        statusMessageElement.textContent = message;
        statusMessageElement.style.color = success ? "green" : "red";
        setTimeout(() => {
            statusMessageElement.textContent = "";
        }, 3000);
    }

    function getDefaultButtonSettings() {
        return Array.from({ length: 10 }, (_, i) => ({
            type: "B",
            text: `リンクB${i + 1}`,
            url: "",
        })).concat(
            Array.from({ length: 10 }, (_, i) => ({
                type: "L",
                text: `リンクL${i + 1}`,
                url: "",
            }))
        ).concat(
            Array.from({ length: 10 }, (_, i) => ({
                type: "R",
                text: `リンクR${i + 1}`,
                url: "",
            }))
        );
    }

    function generateButtonInputs(buttonType, buttonCount, buttonSettings) {
        const buttonTextInputs = document.getElementById(`buttonTextInputs${buttonType}`);
        if (!buttonTextInputs) {
            console.error(`buttonTextInputs${buttonType} not found.`);
            return;
        }

        buttonTextInputs.innerHTML = "";

        for (let i = 1; i <= buttonCount; i++) {
            const buttonTextId = `button${buttonType}${i}Text`;
            const buttonUrlId = `button${buttonType}${i}Url`;

            // テキストとURLの要素を取得
            const buttonTextElement = document.getElementById(buttonTextId);
            const buttonUrlElement = document.getElementById(buttonUrlId);

            // エラーチェックを追加
            if (!buttonTextElement || !buttonUrlElement) {
                console.error(`Element not found for ${buttonTextId} or ${buttonUrlId}`);
                continue; // 要素が見つからない場合はスキップ
            }

            // デバッグログ
            console.log(`Checking elements for ${buttonTextId}`);
            console.log(`Initial values for ${buttonTextId}: ${buttonTextElement.value}, ${buttonUrlId}: ${buttonUrlElement.value}`);

            const inputText = createElement("input", {
                id: buttonTextId,
                type: "text",
            });

            const inputUrl = createElement("textarea", {
                id: buttonUrlId,
                rows: "3",
                cols: "20",
            });

            // 同一の td 要素内にラベル、改行、テキスト、改行 を追加
            const labelCell = createElement("td", {});
            labelCell.appendChild(document.createTextNode(`ボタン${buttonType}${i}: `));
            labelCell.appendChild(document.createElement("p")); // 改行を追加
            labelCell.appendChild(inputText);

            const row = createElement("tr");
            const urlCell = createElement("td");
            urlCell.appendChild(inputUrl);
            row.appendChild(labelCell);
            row.appendChild(urlCell);
            buttonTextInputs.appendChild(row); // 新しいボタンを追加
        }
    }




    chrome.storage.sync.get(
        {
            buttonCountB: 5,
            buttonCountL: 10,
            buttonCountR: 10,
            waitTime: 300,
            buttonSettings: getDefaultButtonSettings(),
            recoveryOption: "on",
        },
        function (data) {
            console.log("Loaded settings from storage:", data);

            document.getElementById("buttonCountB").value = data.buttonCountB;
            document.getElementById("buttonCountL").value = data.buttonCountL;
            document.getElementById("buttonCountR").value = data.buttonCountR;
            document.getElementById("waitTime").value = data.waitTime;

            // ラジオボタンの生成と初期状態の設定
            addRecoveryOptionRadio("回復ON", "on", data.recoveryOption === "on");
            addRecoveryOptionRadio("回復OFF", "off", data.recoveryOption === "off");

            // Use the loaded data.buttonSettings to generate button text & URL fields
            ["B", "L", "R"].forEach((buttonType) => {
                generateButtonInputs(buttonType, data[`buttonCount${buttonType}`], data.buttonSettings);
            });
        }
    );

    // イベントリスナーを設定
    document.getElementById("saveButton").addEventListener("click", function () {
        const recoveryOption = document.querySelector(
            'input[name="recoveryOption"]:checked'
        ).value;
        const waitTime = document.getElementById("waitTime").value;
        const buttonSettings = [];

        const getButtonCount = (buttonType) => {
            const buttonCountElement = document.getElementById(`buttonCount${buttonType}`);
            return buttonCountElement ? parseInt(buttonCountElement.value, 10) : 0;
        };

        const processButtonInputs = (buttonType) => {
            const buttonCount = getButtonCount(buttonType);
            for (let i = 1; i <= buttonCount; i++) {
                const buttonTextElement = document.getElementById(`button${buttonType}${i}Text`);
                const buttonText =
                    buttonTextElement && buttonTextElement.value
                        ? buttonTextElement.value
                        : `リンク${buttonType}${i}`;
                const buttonUrlElement = document.getElementById(`button${buttonType}${i}Url`);
                const buttonUrl = buttonUrlElement && buttonUrlElement.value
                    ? buttonUrlElement.value
                    : "";
                buttonSettings.push({ type: buttonType, text: buttonText, url: buttonUrl });
            }
        };
        const buttonCountB = getButtonCount("B");
        const buttonCountL = getButtonCount("L");
        const buttonCountR = getButtonCount("R");

        processButtonInputs("B");
        processButtonInputs("L");
        processButtonInputs("R");

        chrome.storage.sync.set(
            {
                buttonCountB: buttonCountB,
                buttonCountL: buttonCountL,
                buttonCountR: buttonCountR,
                waitTime: waitTime,
                buttonSettings: buttonSettings,
                recoveryOption: recoveryOption
            },
            function () {
                console.log("Settings saved:", {
                    buttonCountB,
                    buttonCountL,
                    buttonCountR,
                    waitTime,
                    buttonSettings,
                    recoveryOption,
                });
                displayStatusMessage("保存しました", true);
            }
        );
    });
});
